var settings = {
  width: document.body.clientWidth,
  height: document.body.clientHeight,
  cellsize: Math.floor(Math.random() * Math.floor(100)) + 50,
  cellpadding: Math.floor(Math.random() * Math.floor(10)) + 5,
  bleed: Math.floor(Math.random() * Math.floor(200)) + 50,
  fillopacity: 1,
  strokeopacity: 1,
  x_gradient: ["#00FFFC", "#FC00FF", "#fffc00"],
  y_gradient: ["#fffc00", "#FC00FF", "#00FFFC"]
};

settings.cellsX = Math.ceil(
  (settings.width + settings.bleed * 2) / settings.cellsize
);
settings.cellsY = Math.ceil(
  (settings.height + settings.bleed * 2) / settings.cellsize
);

// generate random data set to fill window and a bit beyond
var nodes = d3.range(settings.cellsX * settings.cellsY).map(function (d) {
  var col = d % settings.cellsX;
  var row = Math.floor(d / settings.cellsX);
  var x = Math.round(
    -settings.bleed +
      col * settings.cellsize +
      Math.random() * (settings.cellsize - settings.cellpadding * 2) +
      settings.cellpadding
  );
  var y = Math.round(
    -settings.bleed +
      row * settings.cellsize +
      Math.random() * (settings.cellsize - settings.cellpadding * 2) +
      settings.cellpadding
  );
  return [x, y];
});

var svg = d3
  .select("body")
  .append("svg")
  .attr("id", "voronoi")
  .attr("width", settings.width)
  .attr("height", settings.height)
  .on("mousemove", function () {
    nodes[0] = d3.mouse(this);
    draw();
  });

var voronoi = d3.geom
  .voronoi()
  .clipExtent([
    [settings.cellpadding, settings.cellpadding],
    [
      settings.width - settings.cellpadding,
      settings.height - settings.cellpadding
    ]
  ])
  .triangles(nodes);

var path = svg.append("g").selectAll("path");

function color(x, y) {
  var color_x = d3.scale
    .linear()
    .range(settings.x_gradient)
    .domain(
      d3.range(0, settings.width, settings.width / settings.x_gradient.length)
    ); //[-bleed, width+bleed]
  var color_y = d3.scale
    .linear()
    .range(settings.y_gradient)
    .domain(
      d3.range(0, settings.height, settings.height / settings.y_gradient.length)
    ); //[-bleed, width+bleed]
  return d3.interpolateRgb(color_x(x), color_y(y))(0.5);
}

// draw the triangles
function draw() {
  path = path.data(
    d3.geom
      .voronoi()
      .triangles(nodes)
      .map(function (d) {
        return "M" + d.join("L") + "Z";
      }),
    String
  );

  path
    .enter()
    .append("path")
    .style("fill", function (d, i) {
      var c = d.replace(/M/g, "").replace(/Z/g, "").replace(/L/g, ",");
      var c = c.split(",");
      var x = (parseFloat(c[0]) + parseFloat(c[2]) + parseFloat(c[4])) / 3;
      var y = (parseFloat(c[1]) + parseFloat(c[3]) + parseFloat(c[5])) / 3;
      return color(x, y);
    })
    .style("stroke", function (d, i) {
      var c = d.replace(/M/g, "").replace(/Z/g, "").replace(/L/g, ",");
      var c = c.split(",");
      var x = (parseFloat(c[0]) + parseFloat(c[2]) + parseFloat(c[4])) / 3;
      var y = (parseFloat(c[1]) + parseFloat(c[3]) + parseFloat(c[5])) / 3;
      return color(x, y);
    })
    .style("fill-opacity", function (d, i) {
      if (settings.fillopacity != 1) return settings.fillopacity;
    })
    .style("stroke-opacity", function (d, i) {
      if (settings.strokeopacity != 1) return settings.strokeopacity;
    })
    .attr("d", String);

  path.exit().remove();
}

draw();